import type { DownloadOptions, TransferProgress } from 'koajax';
import type { FileSystemHandle } from 'native-file-system-adapter';
import { ReadableStream } from 'web-streams-polyfill';
import { ByteSize } from 'web-utility';
export declare abstract class DownloadTask implements Partial<TransferProgress> {
    name: string;
    path: string;
    accessor id: string;
    accessor fsHandle: FileSystemHandle | undefined;
    accessor total: number;
    accessor loaded: number;
    accessor percent: number;
    accessor executing: boolean;
    accessor options: DownloadOptions | undefined;
    stream?: ReadableStream<Partial<TransferProgress>>;
    get totalSize(): ByteSize;
    get loadedSize(): ByteSize;
    constructor(name: string, path: string);
    toJSON(): {
        total: number;
        loaded: number;
        percent: number;
        id: string;
        name: string;
        path: string;
        fsHandle: FileSystemHandle;
        options: DownloadOptions;
    };
    saveMeta(data?: Partial<TransferProgress>): Partial<TransferProgress>;
    abstract loadStream(options?: DownloadOptions): AsyncGenerator<Partial<TransferProgress>>;
    pause(): Promise<void>;
    start(options?: DownloadOptions): Promise<ReadableStream<Partial<TransferProgress>>>;
    destroy(): Promise<void>;
    onFinished(callback: (task: DownloadTask) => any): import("mobx").IReactionDisposer;
}
