import { AbstractClass, IndexKey, TypeKeys } from 'web-utility';
import { BaseListModel } from './Base';
import { DataObject, IDType, NewData } from './utility';
export type Filter<T extends DataObject> = Partial<NewData<T>>;
export interface PageData<D extends DataObject> {
    pageData: D[];
    totalCount?: number;
}
export type Statistic<D extends DataObject> = Partial<Record<TypeKeys<D, IndexKey>, Record<string, number>>>;
export declare abstract class ListModel<D extends DataObject, F extends Filter<D> = Filter<D>> extends BaseListModel<D> {
    accessor pageIndex: number;
    accessor pageSize: number;
    accessor filter: F;
    accessor totalCount: number | undefined;
    accessor pageList: D[][];
    get currentPage(): D[];
    get pageCount(): number;
    get allItems(): D[];
    get noMore(): boolean;
    accessor statistic: Statistic<D>;
    clearList(): void;
    clear(): void;
    restoreList({ pageIndex, pageSize, allItems, totalCount }?: Partial<Pick<ListModel<D>, 'pageIndex' | 'pageSize' | 'allItems' | 'totalCount'>>): void;
    turnTo(pageIndex: number, pageSize?: number): ListModel<D, F>;
    /**
     * @protected
     */
    abstract loadPage(pageIndex: number, pageSize: number, filter: F): Promise<PageData<D>>;
    /**
     * @protected
     */
    loadNewPage(pageIndex: number, pageSize: number, filter: F): Promise<{
        pageData: D[];
        totalCount: number;
    }>;
    getList(filter?: F, pageIndex?: number, pageSize?: number): Promise<D[]>;
    refreshList(): Promise<D[]>;
    getAll(filter?: F, pageSize?: number): Promise<D[]>;
    countAll(keys: TypeKeys<D, IndexKey>[], filter?: F, pageSize?: number): Promise<Partial<Record<TypeKeys<D, IndexKey>, Record<string, number>>>>;
    indexOf(id: IDType): number;
    changeOne(data: Partial<D>, id: IDType, patch?: boolean): void;
    updateOne(data: Partial<NewData<D>>, id?: IDType): Promise<D>;
    removeOne(id: IDType): Promise<void>;
    deleteOne(id: IDType): Promise<void>;
}
export declare function Buffer<D extends DataObject, F extends Filter<D> = Filter<D>, M extends AbstractClass<ListModel<D, F>> = AbstractClass<ListModel<D, F>>>(Super: M): (abstract new (...args: any[]) => {
    pendingList: ReturnType<ListModel<D, F>["loadPage"]>[];
    clearList(): void;
    clear(): void;
    getList(filter?: F, pageIndex?: number, pageSize?: number): Promise<D[]>;
    pageIndex: number;
    pageSize: number;
    filter: F;
    totalCount: number;
    pageList: D[][];
    readonly currentPage: D[];
    readonly pageCount: number;
    readonly allItems: D[];
    readonly noMore: boolean;
    statistic: Partial<Record<TypeKeys<D, IndexKey>, Record<string, number>>>;
    restoreList({ pageIndex, pageSize, allItems, totalCount }?: Partial<Pick<ListModel<D, Partial<NewData<D>>>, "pageIndex" | "pageSize" | "totalCount" | "allItems">>): void;
    turnTo(pageIndex: number, pageSize?: number): ListModel<D, F>;
    /**
     * @protected
     */
    loadPage(pageIndex: number, pageSize: number, filter: F): Promise<PageData<D>>;
    /**
     * @protected
     */
    loadNewPage(pageIndex: number, pageSize: number, filter: F): Promise<{
        pageData: D[];
        totalCount: number;
    }>;
    refreshList(): Promise<D[]>;
    getAll(filter?: F, pageSize?: number): Promise<D[]>;
    countAll(keys: TypeKeys<D, IndexKey>[], filter?: F, pageSize?: number): Promise<Partial<Record<TypeKeys<D, IndexKey>, Record<string, number>>>>;
    indexOf(id: IDType): number;
    changeOne(data: Partial<D>, id: IDType, patch?: boolean): void;
    updateOne(data: Partial<NewData<D>>, id?: IDType): Promise<D>;
    removeOne(id: IDType): Promise<void>;
    deleteOne(id: IDType): Promise<void>;
    client: import("./utility").RESTClient;
    baseURI: string;
    indexKey: keyof D;
    currentOne: D;
    clearCurrent(): void;
    getOne(id: IDType): Promise<D>;
    downloading: number;
    uploading: number;
}) & M;
export declare function Stream<D extends DataObject, F extends Filter<D> = Filter<D>, M extends AbstractClass<ListModel<D, F>> = AbstractClass<ListModel<D, F>>>(Super: M): (abstract new (...args: any[]) => {
    baseURI: string;
    stream?: AsyncGenerator<D, void, any>;
    openStream(filter: F): AsyncGenerator<D, void, any>;
    clearList(): void;
    clear(): void;
    restoreList({ filter, pageIndex, pageSize, allItems, totalCount }?: Partial<Pick<ListModel<D>, "filter" | "pageIndex" | "pageSize" | "allItems" | "totalCount">>): Promise<void>;
    /**
     * @protected
     */
    loadStream(filter: F, newCount: number): Promise<D[]>;
    /**
     * @protected
     */
    loadPage(pageIndex: number, pageSize: number, filter: F): Promise<{
        pageData: D[];
        totalCount: number;
    }>;
    pageIndex: number;
    pageSize: number;
    filter: F;
    totalCount: number;
    pageList: D[][];
    readonly currentPage: D[];
    readonly pageCount: number;
    readonly allItems: D[];
    readonly noMore: boolean;
    statistic: Partial<Record<TypeKeys<D, IndexKey>, Record<string, number>>>;
    turnTo(pageIndex: number, pageSize?: number): ListModel<D, F>;
    /**
     * @protected
     */
    loadNewPage(pageIndex: number, pageSize: number, filter: F): Promise<{
        pageData: D[];
        totalCount: number;
    }>;
    getList(filter?: F, pageIndex?: number, pageSize?: number): Promise<D[]>;
    refreshList(): Promise<D[]>;
    getAll(filter?: F, pageSize?: number): Promise<D[]>;
    countAll(keys: TypeKeys<D, IndexKey>[], filter?: F, pageSize?: number): Promise<Partial<Record<TypeKeys<D, IndexKey>, Record<string, number>>>>;
    indexOf(id: IDType): number;
    changeOne(data: Partial<D>, id: IDType, patch?: boolean): void;
    updateOne(data: Partial<NewData<D>>, id?: IDType): Promise<D>;
    removeOne(id: IDType): Promise<void>;
    deleteOne(id: IDType): Promise<void>;
    client: import("./utility").RESTClient;
    indexKey: keyof D;
    currentOne: D;
    clearCurrent(): void;
    getOne(id: IDType): Promise<D>;
    downloading: number;
    uploading: number;
}) & M;
